"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var diff = function (a, b) {
    var pd = a.sortOrder - b.sortOrder;
    if (pd) {
        return pd;
    }
    return a.index - b.index;
};
var Scheduler = (function (_super) {
    __extends(Scheduler, _super);
    function Scheduler() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.queue = [];
        _this.timeout = 0;
        _this.sortOnQ = false;
        _this.state = {
            recalculate: 0,
            queue: _this.queue
        };
        _this.add = function (cb, priority, ref) {
            var q = {
                cb: cb,
                priority: priority,
                index: _this.queue.length,
                sortOrder: priority,
                ref: ref,
                executed: false
            };
            _this.queue.push(q);
            _this.update();
            return q;
        };
        _this.remove = function (q) {
            var index = _this.queue.indexOf(q);
            if (index > 0) {
                _this.queue.splice(index, 1);
            }
        };
        _this.replace = function (q, cb, priority, ref) {
            if (q) {
                var changedPriority = q.priority !== priority;
                var changedRef = q.ref !== ref;
                q.cb = cb;
                q.priority = priority;
                q.ref = ref;
                if (changedPriority || changedRef) {
                    _this.update();
                }
            }
            return q;
        };
        _this.reset = function () {
            _this.queue.forEach(function (q) { return q.executed = false; });
            _this.update();
        };
        _this.channel = {
            add: _this.add,
            remove: _this.remove,
            replace: _this.replace,
            reset: _this.reset,
        };
        return _this;
    }
    Scheduler.prototype.componentWillUnmount = function () {
        window.clearTimeout(this.timeout);
    };
    Scheduler.prototype.componentDidUpdate = function (props) {
        if (this.props.observe !== props.observe ||
            this.props.reverse !== props.reverse ||
            this.props.disabled !== props.disabled) {
            this.update();
        }
    };
    Scheduler.prototype.update = function () {
        this.sortOnQ = true;
        this.scheduleQ(this.props.noInitialDelay && this.noExecutedTask()
            ? 0
            : this.props.stepDelay);
    };
    Scheduler.prototype.scheduleQ = function (when) {
        var _this = this;
        if (!this.timeout && !this.props.disabled) {
            this.timeout = window.setTimeout(function () {
                _this.timeout = 0;
                if (_this.sortOnQ) {
                    _this.sortOnQ = false;
                    _this.sortQ();
                }
                _this.executeQ();
            }, when);
        }
    };
    Scheduler.prototype.sortQ = function () {
        var _this = this;
        if (this.props.source) {
            this.queue.forEach(function (q) { return q.sortOrder = q.priority < Infinity ? _this.props.source(q) : q.priority; });
        }
        else {
            this.queue.forEach(function (q) { return q.sortOrder = q.priority; });
        }
        if (this.props.reverse) {
            this.queue.sort(function (a, b) { return diff(b, a); });
        }
        else {
            this.queue.sort(function (a, b) { return diff(a, b); });
        }
    };
    Scheduler.prototype.nextQ = function () {
        return this.queue.filter(function (_a) {
            var executed = _a.executed, priority = _a.priority, sortOrder = _a.sortOrder;
            return !executed && sortOrder < Infinity && priority < Infinity;
        })[0];
    };
    Scheduler.prototype.executeQ = function () {
        var _this = this;
        var q = this.nextQ();
        if (q) {
            q.executed = true;
            Promise.resolve(q.cb())
                .then(function (result) {
                var dt = (typeof result === "number" ? result : 0) || 0;
                var when = Math.max(0, _this.props.stepDelay + dt);
                _this.scheduleQ(when);
                if (_this.props.withSideEffect) {
                    _this.setState({
                        recalculate: 1
                    });
                }
            });
        }
        else {
            this.props.onEmptyQueue && this.props.onEmptyQueue();
        }
    };
    Scheduler.prototype.noExecutedTask = function () {
        return !this.queue.find(function (x) { return x.executed; });
    };
    Scheduler.prototype.render = function () {
        return this.props.children(this.channel);
    };
    return Scheduler;
}(React.Component));
exports.Scheduler = Scheduler;
